"""
Functionality needed to run a client request sequence (clients.*.main) in the ImageJ model_server 2.7 script environment
"""

import httplib
import json
import urllib

from ij import ImagePlus

HOST = '127.0.0.1'
PORT = 6221
uri = 'http://{}:{}/'.format(HOST, PORT)

def hit_endpoint(method, endpoint, params=None):
    """
    Python 2.7 implementation of HTTP client
    :param method: (str) either 'GET' or 'PUT'
    :param endpoint: (str) endpoint of HTTP request
    :param params: (dict) of parameters required by client request
    :return: (dict) of response status and content, formatted as dict if request is successful
    """
    connection = httplib.HTTPConnection(HOST, PORT)
    if not method in ['GET', 'PUT']:
        raise Exception('Can only handle GET and PUT requests')
    if params:
        url = endpoint + '?' + urllib.urlencode(params)
    else:
        url = endpoint
    connection.request(method, url)
    resp = connection.getresponse()
    resp_str = resp.read()
    try:
        content = json.loads(resp_str)
    except Exception:
        content = {'str': str(resp_str)}
    return {'status': resp.status, 'content': content}

def run_request_sequence(imp, func, params):
    """
    Execute a sequence of client requests in the ImageJ scripting environment
    :param imp: (ij.ImagePlus) input image
    :param func: (func) function that implements client request sequence
    :param params: (dict) parameters specific to client request
    :return: (ij.ImagePlus) output image
    """
    in_path = imp.getProp('Location')
    out_path = func(hit_endpoint, in_path, params)
    return ImagePlus(out_path)